#include "cubemap.h"

CM::CM(FrameBuffer *crossImage) {

  int w = crossImage->w/3, h = w;
  if (crossImage->h/4 != h) {
    cerr << "ERROR: expected a 3:4 aspect ratio in CM::CM(FrameBuffer *crossImage)";
    return;
  }

  int u0, v0;

  phcs[0] = new PHC(w, w, 90.0f);
  u0 = w;
  v0 = w;
  faces[0] = new FrameBuffer(10, 10, w, w);
  faces[0]->Set(crossImage, u0, v0);
  faces[0]->label("front");

  phcs[1] = new PHC(*phcs[0]);
  phcs[1]->Pan(90.0f);
  u0 = 0;
  v0 = w;
  faces[1] = new FrameBuffer(10, 10, w, w);
  faces[1]->Set(crossImage, u0, v0);
  faces[1]->label("left");

  phcs[2] = new PHC(*phcs[0]);
  phcs[2]->Pan(-90.0f);
  u0 = 2*w;
  v0 = w;
  faces[2] = new FrameBuffer(10, 10, w, w);
  faces[2]->Set(crossImage, u0, v0);
  faces[2]->label("right");

  phcs[3] = new PHC(*phcs[0]);
  phcs[3]->Tilt(90.0f);
  u0 = w;
  v0 = 0;
  faces[3] = new FrameBuffer(10, 10, w, w);
  faces[3]->Set(crossImage, u0, v0);
  faces[3]->label("top");

  phcs[4] = new PHC(*phcs[0]);
  phcs[4]->Tilt(-90.0f);
  u0 = w;
  v0 = 2*w;
  faces[4] = new FrameBuffer(10, 10, w, w);
  faces[4]->Set(crossImage, u0, v0);
  faces[4]->label("bottom");

  phcs[5] = new PHC(*phcs[0]);
  phcs[5]->Tilt(180.0f);
  u0 = w;
  v0 = 3*w;
  faces[5] = new FrameBuffer(10, 10, w, w);
  faces[5]->Set(crossImage, u0, v0);
  faces[5]->label("back");

  prevFace = 0;

}

unsigned int CM::LookUp(V3 ray) {

  int i, j;
  for (i = prevFace, j = 0; j < 6; i = (i+1)%6, j++) {
    V3 projP;
    if (phcs[i]->Project(ray, projP)) {
      prevFace = i;
      return faces[i]->Get((int)projP[0], (int) projP[1]);
    }
  }

  return 0xFF0000FF;

}